#include <iostream>

#include <QMessageBox>
#include <QRegExpValidator>

#include "dialog.h"

Dialog::Dialog(bool currPIN, QWidget *parent, Qt::WindowFlags flags) : QDialog(parent, flags)
{
  ui.setupUi(this);

  currentPIN=currPIN;

  QRegExp rx("[0-9]*");
  validator1=validator2=nullptr;

  if (currentPIN) {
    ui.typeNewPINLabel->hide();    ui.typeNewPINLineEdit->hide();
    ui.retypeNewPINLabel->hide();  ui.retypeNewPINLineEdit->hide();

    ui.currentPINLineEdit->setFocus();
    validator1 = new QRegExpValidator(rx, ui.typeNewPINLineEdit);
    ui.currentPINLineEdit->setValidator(validator1);
  }
  else {
    ui.currentPINLabel->hide();    ui.currentPINLineEdit->hide();
    ui.typeNewPINLineEdit->setFocus();

    validator1 = new QRegExpValidator(rx, ui.typeNewPINLineEdit);
    validator2 = new QRegExpValidator(rx, ui.retypeNewPINLineEdit);
    ui.typeNewPINLineEdit->setValidator(validator1);
    ui.retypeNewPINLineEdit->setValidator(validator2);
  }
  this->layout()->invalidate();
  resize(10,10); //to stretch contents
}

Dialog::~Dialog()
{
}

void Dialog::PrintResult()
{
  if (currentPIN) {
        std::cout << ui.currentPINLineEdit->text().toStdString() << std::flush;
  }
  else {
        std::cout << ui.typeNewPINLineEdit->text().toStdString() << std::flush;
  }
}

bool Dialog::CheckResult(bool printMessage)
{
  if (currentPIN) {
        if (ui.currentPINLineEdit->text().isEmpty()) {
          if (printMessage) QMessageBox::critical(this, tr("Attention!"), tr("PIN is empty!"), QMessageBox::Ok);
          return false;
        }
  }
  else {
        if (ui.typeNewPINLineEdit->text().isEmpty()) {
             if (printMessage) QMessageBox::critical(this, tr("Attention!"), tr("PIN is empty!"), QMessageBox::Ok);
             return false;
        }
        else if (ui.typeNewPINLineEdit->text()!=ui.retypeNewPINLineEdit->text()) {
             if (printMessage) QMessageBox::critical(this, tr("Attention!"), tr("PINs are different!"), QMessageBox::Ok);
             return false;
        }
  }
  return true;
}
