#include "exceptions.h"

#include <QApplication>


DetailsException::DetailsException()
{
}

DetailsException::~DetailsException()
{
}


CommonException::CommonException(const QString& details) :
    m_details(details)
{
}

CommonException::~CommonException()
{
}

const QString CommonException::fullDetails() const
{
    return m_details;
}


FileException::FileException(const QString& path) :
    m_path(path)
{
}

FileException::~FileException()
{
}

const QString FileException::path() const
{
    return m_path;
}


OpenFileException::OpenFileException(const QString& path,
                             const QIODevice::OpenMode mode,
                             const QString& details) :
    FileException(path),
    m_mode(mode),
    m_details(details)
{
}

OpenFileException::~OpenFileException()
{
}

const QIODevice::OpenMode OpenFileException::mode() const
{
    return m_mode;
}

const QString OpenFileException::details() const
{
    return m_details;
}

const QString OpenFileException::fullDetails() const
{
    return QString("%1. %2.").arg(QApplication::tr("Can not open file '%1'").arg(m_path)).arg(m_details);
}


FileNotExistsException::FileNotExistsException(const QString& path) :
    FileException(path)
{
}

FileNotExistsException::~FileNotExistsException()
{
}

const QString FileNotExistsException::fullDetails() const
{
    return QApplication::tr("File '%1' not exists").arg(m_path);
}


PluginLoadException::PluginLoadException(const QString& path, const QString& details) :
    CommonException(details),
    m_path(path)
{
}

PluginLoadException::~PluginLoadException()
{
}

const QString PluginLoadException::path() const
{
    return m_path;
}

const QString PluginLoadException::fullDetails() const
{
    return QApplication::tr("Can not load '%1' plugine. %2").arg(m_path).arg(m_details);
}
