#ifndef EXCEPTIONS_H
#define EXCEPTIONS_H

#include "dm_common_global.h"

#include <QException>
#include <QIODevice>


class DM_COMMONSHARED_EXPORT DetailsException : public QException {
public:
    explicit DetailsException();
    virtual ~DetailsException();

public:
    virtual const QString fullDetails() const = 0;
};


class DM_COMMONSHARED_EXPORT CommonException : public DetailsException {
public:
    explicit CommonException(const QString& details);
    ~CommonException() override;

public:
    const QString fullDetails() const override;

protected:
    QString m_details;
};


class DM_COMMONSHARED_EXPORT FileException : public DetailsException {
public:
    explicit FileException(const QString& path);
    virtual ~FileException();

public:
    const QString path() const;

protected:
    QString m_path;
};


class DM_COMMONSHARED_EXPORT OpenFileException : public FileException
{
public:
    explicit OpenFileException(const QString& path,
                               const QIODevice::OpenMode mode,
                               const QString& details);
    ~OpenFileException() override;

public:
    const QIODevice::OpenMode mode() const;
    const QString details() const;
    const QString fullDetails() const override;

private:
    QIODevice::OpenMode m_mode;
    QString m_details;
};


class DM_COMMONSHARED_EXPORT FileNotExistsException : public FileException {
public:
    explicit FileNotExistsException(const QString& path);
    ~FileNotExistsException() override;

public:
    const QString fullDetails() const override;
};


class DM_COMMONSHARED_EXPORT PluginLoadException : public CommonException
{
public:
    explicit PluginLoadException(const QString& path, const QString& details);
    ~PluginLoadException() override;

public:
    const QString path() const;
    const QString fullDetails() const override;

private:
    QString m_path;
};

#endif // EXCEPTIONS_H
