#ifndef DOMAINMANAGER_H
#define DOMAINMANAGER_H

#include <QObject>
#include <QProcess>
#include <QStringList>
#include <QFile>
#include <QTextStream>
#include <QSettings>

#include "localauthmanager.h"
#include "dm_common_global.h"

struct DomainSettings {
    QStringList domains;
    QString defaultDomain; //TODO handle default domain here or anywhere esle?
    QString separator;
    bool isPrefixDomain;
};

class DM_COMMONSHARED_EXPORT DomainManager : public QObject
{
    Q_OBJECT
public:
    explicit DomainManager(const DomainSettings &settings = DomainSettings(), QObject *parent = nullptr);

    bool inDomain() const;

private:
    DomainSettings m_settings;
    QStringList m_staticDomains;
    QStringList m_domains;
    QStringList m_domainsScripts;
    QProcess m_domainsProcess;
    const QString m_localDomainName = tr("Local computer");
    bool m_inDoimain = false;

signals:
    void loginRequestDataHandled(const AuthData& data);
    void domainsListChanged(const QStringList &domains);
    void errorOccurred();

public slots:
    void handleLoginRequestData(const AuthData& data, const QString &domain);
    void updateDomainsList(int exitCode, QProcess::ExitStatus exitStatus);
    void startGetDomainsScript();
};

#endif // DOMAINMANAGER_H
