#ifndef LOCALAUTHMANAGER_H
#define LOCALAUTHMANAGER_H

#include "dm_common_global.h"

#include <QObject>
#include <QString>

#include "manager.h"


struct AuthData {
    QString username;
    QString password;
};


class DM_COMMONSHARED_EXPORT LocalAuthManager : public QObject, public Manager
{
    Q_OBJECT

public:
    explicit LocalAuthManager(const int autoLoginDelay = 0,
                              const QString& autoLoginUser = QString(),
                              QObject* parent = nullptr);
    ~LocalAuthManager() override;

public:
    bool autoLoginEnabled() const;
    int autoLoginDelay() const;
    const QString autoLoginUser() const;
    const QString hostname() const;
    void setHostname(const QString &hostname);
    bool focusPassword() const;
    void setFocusPassword(bool focusPassword);

signals:
    void loginRequest(const AuthData& data);
    void autoLoginRequest();
    void loginFailed();
    void successLogin();
    void passwordChangeRequested(const bool withAuth);
    void newPasswordInsertionCompleted(const QString &login,
                                       const QString &oldPassword,
                                       const QString &newPassword,
                                       const QString &confirmPassword);
    void newPasswordInsertionRejected();
    void pamErrorReceived(const QString &message, const bool isPasswordChange);
    void messageReceived(const QString &message);
    void requestReceived(const QString &request, bool echo);
    void enableGenericMode();
    void sendAnswerToPam(const QString &answer);

public slots:
    void performLoginRequest(const AuthData& data);
    void performAutoLoginRequest();

private:
    int m_autoLoginDelay;
    QString m_autoLoginUser;
    QString m_hostname;
    bool m_focusPassword;
};

#endif // LOCALAUTHMANAGER_H
