#ifndef USERSMANAGER_H
#define USERSMANAGER_H

#include "dm_common_global.h"

#include <QObject>

#include "manager.h"
#include "userinfo.h"
#include "structures.h"
#include "stssettings.h"


class DM_COMMONSHARED_EXPORT UsersManager : public QObject, public Manager
{
    Q_OBJECT

public:
    enum PreselUser {
        None = 0,
        Prev,
        Default
    };

    enum FirstScreen {
        Auto = 0,
        Shortcuts,
        TextEnter
    };

public:
    explicit UsersManager(const UsersPwdList& users,
                          const IconList& icons,
                          StsSettings* stsSettings,
                          const bool showTokenUsers,
                          const bool showUsers,
                          const QString defaultUser,
                          const PreselUser preselUser,
                          const bool userCompletion,
                          const FirstScreen firstScreen,
                          QObject* parent = nullptr);
    ~UsersManager() override;

public:
    const UsersData users() const;
    PreselUser preselUser() const;
    const QString prevUsername() const;
    void savePrevUsername(const QString& username) const;
    const QString defaultUser() const;
    static const QString getHostname();
    bool userCompletion() const;
    const bool showTokenUsers;
    const bool showUsers;
    FirstScreen firstScreen() const;

private:
    UsersData m_users;
    UsersData m_tokenUsers;
    StsSettings* m_stsSettings;
    QString m_defaultUser;
    PreselUser m_preselUser;
    bool m_userCompletion;
    FirstScreen m_firstScreen;

signals:
    void usersChanged(const UsersData &users);
    void askPamForLastUserSession(const QString& username);
    void updateLastUserSession(const QString &sessionType);

private slots:
    void tokenUsersChanged(const QStringList &tokenUsersList);
};

#endif // USERSMANAGER_H
