#include "virtualkeyboardmanager.h"

#include "process.h"


VirtualKeyboardManager::VirtualKeyboardManager(const QString& command, const int dpi, QObject* parent) :
    QObject(parent),
    Manager("virtualkeyboard")
{
    const auto programCommand = ProgramCommand::fromString(command);

    m_process.setProgram(programCommand.program);
    m_process.setArguments(programCommand.arguments);
    m_process.setEnvironment(QProcess::systemEnvironment() +
                             QStringList() << "QT_SCALE_FACTOR=1"
                                           << QString("QT_FONT_DPI=%1").arg(dpi)); // TODO

    connect(&m_process, &QProcess::errorOccurred,
            this, &VirtualKeyboardManager::errorOccurredHandle);
}

VirtualKeyboardManager::~VirtualKeyboardManager()
{
    close();
}

void VirtualKeyboardManager::open()
{
    close();
    m_process.start();
}

void VirtualKeyboardManager::close()
{
    if (m_process.isOpen()) {
        blockSignals(true);
        m_process.close();
        m_process.kill();
        blockSignals(false);
    }
}

void VirtualKeyboardManager::errorOccurredHandle(const QProcess::ProcessError error)
{
    Q_UNUSED(error);

    emit errorOccurred(m_process.errorString());
}
