#include "sessiondescription.h"

#include <QFile>
#include <QFileInfo>
#include <QTextCodec>
#include <QDir>

#include "exceptions.h"


SessionReader::SessionReader()
{
}

SessionReader::~SessionReader()
{
}

const SessionList SessionReader::readDirs(const char** paths)
{
    QStringList list;
    for (const char** point = paths; *point; ++point) {
        list << QString::fromUtf8(*point);
    }

    return readDirs(list);
}

const SessionList SessionReader::readDirs(const QStringList& paths)
{
    SessionList sessions;

    foreach (const auto& path, paths) {
        foreach (const auto& info, QDir(path).entryInfoList(QDir::Files)) {
            if (info.completeSuffix() == "desktop") {
                sessions << create(info.absoluteFilePath());
            }
        }
    }

    return sessions;
}

const SessionDescription SessionReader::create(const QString& path)
{
    QFile file(path);
    if (!file.exists()) {
        throw FileNotExistsException(path);
    }

    SessionDescription description;
    description.type = QFileInfo(path).baseName();

    QSettings settings(path, QSettings::IniFormat);
    settings.beginGroup("Desktop Entry");
    settings.setIniCodec(QTextCodec::codecForLocale());

    description.name = readLocaleStrings(settings, "Name", QRegularExpression("Name\\s*\\[(?P<locale>\\w+)\\]"));
    description.comment = readLocaleStrings(settings, "Comment", QRegularExpression("Comment\\s*\\[(?P<locale>\\w+)\\]"));
    description.hidden = settings.value("Hidden", false).toBool();
    description.exec = settings.value("Exec", QString()).toString();
    description.tryExec = settings.value("TryExec", QString()).toString();
    description.icon = settings.value("Icon", QString()).toString();

    return description;
}

const SessionDescription::LocaleStrings SessionReader::readLocaleStrings(const QSettings& settings,
                                                                         const QString& nameKey,
                                                                         const QRegularExpression& regExp)
{
    SessionDescription::LocaleStrings strings;

    foreach (const auto& key, settings.allKeys()) {
        if (key == nameKey) {
            strings[""] = settings.value(key, QString()).toString();
            continue;
        }

        const auto match = regExp.match(key);
        if (match.hasMatch()) {
            strings[match.captured("locale")] = settings.value(key, QString()).toString();
        }
    }

    return strings;
}
