#include "swipegesturerecognizer.h"

#include <QSwipeGesture>
#include <QtMath>


SwipeGestureRecognizer::SwipeGestureRecognizer() :
    m_backlash(50)
{
}

SwipeGestureRecognizer::~SwipeGestureRecognizer()
{
}

QGesture* SwipeGestureRecognizer::create(QObject* target)
{
    return new QSwipeGesture(target);
}

QGestureRecognizer::Result SwipeGestureRecognizer::recognize(QGesture* state, QObject* object, QEvent* event)
{
    Q_UNUSED(object);

    const auto mouseEvent = dynamic_cast<QMouseEvent*>(event);
    if (!mouseEvent) {
        return Ignore;
    }

    if(mouseEvent->type() == QMouseEvent::MouseButtonPress) {
        if(const auto gesture = dynamic_cast<QSwipeGesture*>(state)) {
            m_startpoint = mouseEvent->pos();
            gesture->setSwipeAngle(0);
            return TriggerGesture;
        }
    }

    if(mouseEvent->type() == QMouseEvent::MouseButtonRelease) {
        if (mouseEvent->pos() == m_startpoint) {
            return CancelGesture;
        }

        auto point = mouseEvent->pos() - m_startpoint;
        if (qAbs(point.x()) < m_backlash) {
            point.setX(0);
        }
        if (qAbs(point.y()) < m_backlash) {
            point.setY(0);
        }
        if (point.x() == 0 && point.y() == 0) {
            return CancelGesture;
        }

        if(const auto gesture = dynamic_cast<QSwipeGesture*>(state)) {
            auto angel = -qRadiansToDegrees(qAtan2(point.y(), point.x()));
            if (angel < 0) {
                angel += 360;
            }
            gesture->setSwipeAngle(angel);
            return FinishGesture;
        }
    }

    return Ignore;
}
