#include "themedescription.h"

#include <QFile>
#include <QSettings>
#include <QFileInfo>
#include <QDir>

#include "exceptions.h"


ThemeDescription::ThemeDescription(const QString& path) :
    m_path(path)
{
}

ThemeDescription::~ThemeDescription()
{
}

void ThemeDescription::read()
{
    if (!QFile::exists(m_path)) {
        throw FileNotExistsException(m_path);
    }

    QSettings settings(m_path, QSettings::IniFormat);
    settings.beginGroup("KdmGreeterTheme");
    m_greeter = settings.value("Greeter").toString();
    m_plugin = settings.value("Plugin").toString();
    m_widgets = settings.value("Widgets").toStringList();
}

const QString ThemeDescription::path() const
{
    return m_path;
}

const QString ThemeDescription::dirPath() const
{
    return QFileInfo(m_path).dir().absolutePath();
}

const QString ThemeDescription::greeter() const
{
    return m_greeter;
}

const QString ThemeDescription::plugin() const
{
    return m_plugin;
}

const QStringList ThemeDescription::widgets() const
{
    return m_widgets;
}
