#include "themesettings.h"

#include <yaml-cpp/yaml.h>

#include <QDebug>


ThemeSettings::ThemeSettings()
{
}

ThemeSettings::~ThemeSettings()
{
}

void ThemeSettings::load(const QString& path)
{
    m_config = YAML::LoadFile(path.toStdString());
}

const ThemeSettings::BackgroundData ThemeSettings::background() const
{
    BackgroundData data;

    const auto background = m_config["background"];
    if (background.IsDefined()) {
        data.defined = true;
        const auto path = background["path"];
        if (path.IsDefined()) {
            data.path.defined = true;
            data.path.value = QString::fromStdString(path.as<std::string>());
        }

        const auto colorOverlay = background["color_overlay"];
        if (colorOverlay.IsDefined()) {
            data.colorOverlay.defined = true;
            const auto values = QList<int>::fromStdList(colorOverlay.as<std::list<int>>());
            if (values.count() == 4) {
                data.colorOverlay.value = QColor(values[0], values[1], values[2], values[3]);
            } else if (values.count() == 3) {
                data.colorOverlay.value = QColor(values[0], values[1], values[2]);
            } else {
                qDebug() << "Invalid colorOverlay value: " << values;
            }
        }

        const auto blur = background["blur"];
        if (blur.IsDefined()) {
            data.blur.defined = true;
            const auto enable = blur["enable"];
            if (enable.IsDefined()) {
                data.blur.enable.defined = true;
                data.blur.enable.value = enable.as<bool>();
            }

            const auto radius = blur["radius"];
            if (radius.IsDefined()) {
                data.blur.radius.defined = true;
                data.blur.radius.value = radius.as<double>();
            }
        }

        const auto logo = background["logo"];
        if (logo.IsDefined()) {
            data.logo.defined = true;
            const auto enable = logo["enable"];
            if (enable.IsDefined()) {
                data.logo.enable.defined = true;
                data.logo.enable.value = enable.as<bool>();
            }

            const auto path = logo["path"];
            if (path.IsDefined()) {
                data.logo.path.defined = true;
                data.logo.path.value = QString::fromStdString(path.as<std::string>());
            }
        }
    }

    return data;
}
