#include "themewidgetcreator.h"

#include <QFile>
#include <QtUiTools/QUiLoader>
#include <QPluginLoader>
#include <QDir>

#include "exceptions.h"
#include "widgets/basethemewidget.h"
#include "managers/greetermanagers.h"
#include "utils.h"
#include "systeminfo.h"


BaseThemeWidget* ThemeWidgetCreator::create(const ThemeInfo& info,
                                            const QString& widgetPluginName,
                                            GreeterManagers* managers,
                                            const EnvironmentData& enviroment)
{
    const auto pluginePath = SystemInfo::fullPluginPath(widgetPluginName);
    QPluginLoader loader(pluginePath);
    loader.setLoadHints(loader.loadHints() | QLibrary::PreventUnloadHint);
    if (!loader.load()) {
        throw CommonException(tr("'%1' greeter plugine not supported '%2' theme. %3")
                              .arg(widgetPluginName)
                              .arg(info.name())
                              .arg(loader.errorString()));
    }

    const auto plugin = loader.instance();
    const auto themeWidgetPlugin = qobject_cast<AbstractPluginThemeLoader*>(plugin);
    if (!themeWidgetPlugin) {
        ///throw CommonException(tr("'%1' cannot be converted to greeter plugine for '%2'").arg(widgetPluginName).arg(info.name()));
    }
//    const auto themeWidget = qobject_cast<BaseThemeWidget*>(plugin);
    const auto themeWidget = themeWidgetPlugin->createTheme();

    themeWidget->load(info.description(), managers, enviroment);

    return themeWidget;
}

QWidget* ThemeWidgetCreator::create(const QString& path)
{
    QUiLoader uiLoader;
    uiLoader.setWorkingDirectory(QFileInfo(path).dir().absolutePath());

    const auto file = Utils::openFile(path);

    const auto widget = uiLoader.load(file);

    file->close();
    delete file;

    return widget;
}

QWidget* ThemeWidgetCreator::create(QFile* file)
{
    file->seek(0);

    QUiLoader uiLoader;
    uiLoader.setWorkingDirectory(QFileInfo(*file).dir().absolutePath());

    const auto widget = uiLoader.load(file);

    return widget;
}
