#include "tokenwatcher.h"

#include <QFile>

TokenWatcher *TokenWatcher::m_instance = nullptr;

const QString tokenDir = QStringLiteral("/etc/X11/fly-dm/");
const QString tokenPath = QStringLiteral("/etc/X11/fly-dm/token");

TokenWatcher *TokenWatcher::instance()
{
    if (!m_instance) {
        m_instance = new TokenWatcher();
    }

    return m_instance;
}

const QStringList TokenWatcher::init()
{
    connect(&m_watcher, &QFileSystemWatcher::directoryChanged, this, &TokenWatcher::dirChanged);
    connect(&m_watcher, &QFileSystemWatcher::fileChanged, this, &TokenWatcher::fileChanged);

    QStringList tokenUsers;

    if (QFile::exists(tokenPath)) {
        m_watcher.addPath(tokenPath);

        tokenUsers = readToken();
    }
    else {
        m_watcher.addPath(tokenDir);
    }

    return tokenUsers;
}

TokenWatcher::TokenWatcher(QObject *parent) : QObject(parent)
{

}

const QStringList TokenWatcher::readToken()
{
    QStringList tokenUsers;

    QFile token(tokenPath);
    token.open(QFile::ReadOnly);
    tokenUsers = QString::fromLocal8Bit(token.readAll()).split(QStringLiteral("\n"), QString::SkipEmptyParts);
    token.close();

    return tokenUsers;
}

void TokenWatcher::dirChanged()
{
    QStringList tokenUsers;

    if (QFile::exists(tokenPath)) {
        m_watcher.removePath(tokenDir);
        m_watcher.addPath(tokenPath);

        tokenUsers = readToken();
    }

    emit tokenUsersChanged(tokenUsers);
}

void TokenWatcher::fileChanged()
{
    QStringList tokenUsers;

    if (QFile::exists(tokenPath)) {
        tokenUsers = readToken();
    }
    else {
        m_watcher.removePath(tokenPath);
        m_watcher.addPath(tokenDir);
    }

    emit tokenUsersChanged(tokenUsers);
}
