#ifndef USERINFO_H
#define USERINFO_H

#include "dm_common_global.h"

#include <QString>
#include <QList>
#include <QIcon>
#include <QSet>

struct passwd;


struct PasswdLine
{
public:
    static const PasswdLine fromPasswd(passwd* pwd);

public:
    QString username;
    QString password;
    __uid_t uid;
    __uid_t gid;
    QString gecos;
    QString homeDir;
    QString shell;
};


using UsersPwdList = QList<PasswdLine>;


class UserList {
  public:
    UserList(const QStringList& members);
    bool hasUser(const QString& username) const;
    bool hasGroup(const gid_t gid) const;
    bool hasGroups() const;

  private:
    QSet<gid_t> m_groups;
    QSet<QString> m_users;
};


namespace UserInfo DM_COMMONSHARED_EXPORT
{
    struct Filter
    {
        bool showUsers;
        __uid_t lowUserId;
        __uid_t highUserId;
        QStringList users;
        QStringList noUsers;
        bool showRoot;
        bool sortUsers;
    };

    const UsersPwdList  getUserList();
    const UsersPwdList getUserList(const Filter& filter);
}


using IconList = QMap<QString, QIcon>;


namespace UserIcon DM_COMMONSHARED_EXPORT {
    const IconList getUserIcons(const QString& iconsPath);
}

#endif // USERINFO_H
