#include "backgroundwidget.h"

#include <QPainter>
#include <QDebug>

#include <fly/flyfileutils.h>

#include "utils.h"


BackgroundWidget::BackgroundWidget(QWidget* parent) :
    QWidget(parent),
    m_blurRadius(0.0)
{
}

BackgroundWidget::~BackgroundWidget()
{
}

void BackgroundWidget::setImage(const QPixmap& pixmap)
{
    m_imagePixmap = pixmap;
}

void BackgroundWidget::setLogo(const QPixmap& pixmap)
{
    m_logoPixmap = pixmap;
}

void BackgroundWidget::setColor(const QColor& color)
{
    m_color = color;
}

void BackgroundWidget::setBlurRadius(const double radius)
{
    m_blurRadius = radius;
}

void BackgroundWidget::paintEvent(QPaintEvent* event)
{
    QPainter painter(this);
    painter.drawImage(0, 0, m_cachedImage);

    QWidget::paintEvent(event);
}

void BackgroundWidget::resizeEvent(QResizeEvent *event)
{
    updateCachedImage();
    QWidget::resizeEvent(event);
}

void BackgroundWidget::updateCachedImage()
{
    QPixmap pixmap(size());
    QPainter painter(&pixmap);

    if (!m_imagePixmap.isNull()) {
        painter.drawPixmap(0, 0, m_imagePixmap.scaled(size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
    }
    if (!m_logoPixmap.isNull()) {
        // from classic theme code
        const QRect dr = Utils::screenGeometry();
        const auto dpr = (painter.device())?painter.device()->devicePixelRatioF():1.0;
        int x=0,y=0;
        QRect logoRect(x,y,(int)(m_logoPixmap.size().width()),(int)(m_logoPixmap.size().height()));
        float scaleX=1.0, scaleY=1.0, scale=1.0;
        if (logoRect.width()  > dr.width())  scaleX=(float)dr.width()/logoRect.width();
        if (logoRect.height() > dr.height()) scaleY=(float)dr.height()/logoRect.height();
        scale = scaleX < scaleY ? scaleX : scaleY;
        if (scale!=1.0) {
            logoRect.setWidth((int) (logoRect.width()*scale));
            logoRect.setHeight((int)(logoRect.height()*scale));
        }
        CalcLogoPosition(logoRect.width(),logoRect.height(), dr.x(),dr.y(),dr.width(),dr.height(), std::string("BottomRight"), &x,&y);
        logoRect = QRect((int)(x/dpr), (int)(y/dpr), logoRect.width(),logoRect.height());

        painter.drawPixmap(logoRect.x(), logoRect.y(), m_logoPixmap.scaled(logoRect.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
    }
    if (m_color.isValid()) {
        painter.setBrush(m_color);
        painter.drawRect(rect().adjusted(-1, -1, 1, 1));
    }

    if (m_blurRadius > 0) {
        const auto image = pixmap.toImage();
        m_cachedImage = Utils::blurred(image, image.rect(), m_blurRadius, false);
    } else {
        m_cachedImage = pixmap.toImage();
    }
}
