#ifndef COMMONWIDGET_H
#define COMMONWIDGET_H

#include "dm_common_global.h"

#include <QWidget>
#include <QLabel>
#include <QToolButton>

#include "sessiondescription.h"


class DM_COMMONSHARED_EXPORT CommonWidget : public QWidget
{
    Q_OBJECT

public:
    explicit CommonWidget(QWidget *parent = nullptr);
    ~CommonWidget() override;

signals:
    void openVirtualKeyboardRequest();
    void suspendRequest();
    void rebootRequest();
    void shutdownRequest();
    void changeLayoutRequest();
    void currentSessionTypeChanged(const QString& type);
    void aboutToShowAvailibleSessionsMenu();

public:
    void load(QWidget* sourceWidget);
    void enableLoginFailedWidget(const bool enable = true);
    void enableShutdownWidgets(const bool enable = true);
    void enableActions(const bool enable = true);
    void enableSessions(const bool enable = true);
    void setAvailableSessions(const SessionList& availableSessions);
    void setActions(const QList<QAction*>& actions, const QList<QMenu *> &menus = QList<QMenu *>());
    void showActionsMenu(const bool show = true);
    const QString selectedSessionType() const;
    const QString mainWidgetName() const;
    void resetFocus(); // TODO

    QToolButton* m_actionsToolButton;

public slots:
    void changeLayout(const QString& layout);
    void updateLastUserSession(const QString &sessionType);

private:
    const QList<QAction*> createSessionActions();

    bool eventFilter(QObject* watched, QEvent* event) override;

private slots:
    void sessionsMenuActionTriggered(QAction* action);

private:
    QString m_mainWidgetName;
    SessionList m_availableSessions;

    QLabel* m_loginFailedLabel;
    QToolButton* m_virtualKeyboardToolButton;
    QToolButton* m_suspendToolButton;
    QToolButton* m_rebootToolButton;
    QToolButton* m_shutdownToolButton;
    QToolButton* m_layoutToolButton;
    QToolButton* m_sessionsToolButton;
    QWidget* m_menusWidget;
};

#endif // COMMONWIDGET_H
