#include "datetimewidgethandler.h"

#include "utils.h"

#include <QDateTime>


DateTimeWidgetHandler::DateTimeWidgetHandler(QObject* parent) :
    QObject(parent),
    m_dateFormat("ddd, d MMM"),
    m_timeFormat("HH:mm"),
    m_dateRemoveRegExp("\\.+$"),
    m_dateToUpper(true)
{
    m_timer.setInterval(1000);
    connect(&m_timer, &QTimer::timeout,
            this, &DateTimeWidgetHandler::timerTimeout);
}

DateTimeWidgetHandler::~DateTimeWidgetHandler()
{
}

void DateTimeWidgetHandler::load(QWidget* sourceWidget)
{
    m_dateLabel = Utils::findWidget<QLabel*>("dateLabel", sourceWidget);
    m_timeLabel = Utils::findWidget<QLabel*>("timeLabel", sourceWidget);

    update();
}

void DateTimeWidgetHandler::setDateFormat(const QString& format)
{
    m_dateFormat = format;
}

void DateTimeWidgetHandler::setDateToUpper(const bool toUpper)
{
    m_dateToUpper = toUpper;
}

void DateTimeWidgetHandler::setDateRemoveRegExp(const QRegularExpression& regExp)
{
    m_dateRemoveRegExp = regExp;
}

void DateTimeWidgetHandler::setTimeFormat(const QString& format)
{
    m_timeFormat = format;
}

void DateTimeWidgetHandler::start()
{
    m_timer.start();
}

void DateTimeWidgetHandler::stop()
{
    m_timer.stop();
}

void DateTimeWidgetHandler::update()
{
    const auto currentDateTime = QDateTime::currentDateTime();
    updateDate(currentDateTime.date());
    updateTime(currentDateTime.time());
}

void DateTimeWidgetHandler::timerTimeout()
{
    update();
}

void DateTimeWidgetHandler::updateDate(const QDate& date)
{
    auto text = date.toString(m_dateFormat);
    if (m_dateToUpper) {
        text = text.toUpper();
    }
    text = text.remove(m_dateRemoveRegExp);
    m_dateLabel->setText(text);
}

void DateTimeWidgetHandler::updateTime(const QTime& time)
{
    m_timeLabel->setText(time.toString(m_timeFormat));
}
