#include "usercellwidget.h"

#include "utils.h"

#include <QGridLayout>


UserCellWidget::UserCellWidget(const UserData& data, QWidget* parent) :
    QWidget(parent),
    m_data(data)
{
}

UserCellWidget::~UserCellWidget()
{
}

void UserCellWidget::load(QWidget* widget)
{
    const auto layout = new QGridLayout();
    layout->setMargin(0);
    layout->addWidget(widget);
    setLayout(layout);

    m_iconLabel = Utils::findWidget<QLabel*>("iconLabel", widget);
    m_usernameLabel = Utils::findWidget<QLabel*>("usernameLabel", widget);
    m_hostnameLabel = Utils::findWidget<QLabel*>("hostnameLabel", widget);
    m_isLoggedLabel = Utils::findWidget<QLabel*>("isLoggedLabel", widget);

    if (m_data.isTokenUser) {
        setIcon(m_data.icon, m_data.isTokenUser, QIcon::fromTheme("media-flash"));
    }
    else {
        setIcon(m_data.icon);
    }

    setUsername(m_data.username);
    setHostname(m_data.hostname);
    setLogged(m_data.isLoggerd);
}

const UserData UserCellWidget::userData() const
{
    return m_data;
}

void UserCellWidget::setIcon(const QIcon &icon, const bool withEmblem, const QIcon &emblem)
{
    const auto labelSize = Utils::caclulateRealPaintDeviceSize(m_iconLabel);

    if (withEmblem) {
        m_iconLabel->setPixmap(Utils::circlePixmapFromIconWithEmblem(icon, labelSize, emblem));
    }
    else {
        m_iconLabel->setPixmap(Utils::circlePixmapFromIcon(icon, labelSize));
    }
}

void UserCellWidget::setUsername(const QString& username)
{
    m_usernameLabel->setText(username);
}

void UserCellWidget::setHostname(const QString& hostname)
{
    m_hostnameLabel->setText(hostname);
}

void UserCellWidget::setLogged(const bool logged)
{
    m_isLoggedLabel->setVisible(logged);
}
