#include "widgetmenu.h"


WidgetMenu::WidgetMenu(QWidget* parent, QToolButton* toolButton) :
    QMenu(parent),
    m_toolButton(toolButton)
{
    setWindowFlags((windowFlags() ^ Qt::Window) | Qt::Widget);
    hide();

    connect(this, &QMenu::triggered,
            this, &WidgetMenu::hide);

    connect(m_toolButton, &QToolButton::pressed,
            this, &WidgetMenu::toolButtonPressed);

    connect(this, &QMenu::aboutToHide,
            this, &WidgetMenu::performAboutToHide);
}

WidgetMenu::~WidgetMenu()
{
}

void WidgetMenu::keyPressEvent(QKeyEvent* event)
{
    const auto key = event->key();

    if (key == Qt::Key_Tab || key == Qt::Key_Backtab) {
        hide();
        m_toolButton->setFocus();
    }

    QMenu::keyPressEvent(event);
}

void WidgetMenu::toolButtonPressed()
{
    if (isHidden()) {
        m_toolButton->setArrowType(Qt::UpArrow);
        m_toolButton->showMenu();
    } else {
        hide();
    }
}

void WidgetMenu::performAboutToHide()
{
    for (auto action : actions()) {
        const auto submenu = action->menu();

        if (submenu && !submenu->isHidden()) {
            submenu->hide();
        }
    }

    m_toolButton->setArrowType(Qt::DownArrow);
}
