#ifndef GREETERHANDLER_H
#define GREETERHANDLER_H

#include <QSocketNotifier>

#include "managers/localauthmanager.h"
#include "structures.h"
#include "kdmshutdown.h"


class GreeterHandler : public QObject
{
    Q_OBJECT

public:
    explicit GreeterHandler(const int dmSocket, QObject* parent = nullptr);
    virtual ~GreeterHandler();

signals:
    void successLogin();
    void loginFailed();
    void passwordChangeRequestedByPam(const bool withAuth);
    void pamErrorReceived(const QString &message, const bool isPasswordChangeError);
    void messageReceived(const QString &message);
    void requestReceived(const QString &message, bool echo);
    void displaySessionsUpdated(const DpyDescriptions &sessions);

public:
    void start();
    void stop();
    static const QString readLastUserSession(const QString& username);
    static const DpyDescriptions getDisplaySessions(const int flags);
    void doShutdown(const int type, const QString& os = QString());

public slots:
    void handleLoginRequest(const AuthData& data);
    void handleAutoLoginRequest();
    void setSessionType(const QString& session);
    void handleConsoleLoginRequest();
    void switchDisplay(const int vt);
    void onNewPasswordInsertionCompleted(const QString &login,
                                         const QString &oldPassword,
                                         const QString &newPassword,
                                         const QString &confirmPassword);
    void updateDisplaySessions();

    void enableGenericMode();
    void sendAnswerToPam(const QString &answer);

protected slots:
    void handleSocket();

protected:
    QSocketNotifier m_socketNotifier;

private:
    int m_passwordChangeStage = -1;
    int m_echo = -1;

    bool m_passwordChangeWithAuth = false;
    bool m_startedPasswordChange = false;
    bool m_lastGetTextPamEcho = false;
    bool m_isLoginRequested = false;
    bool m_shouldSendPassword = false;
    bool m_shouldSendLogin = false;

    QString m_login;
    QString m_oldPassword;
    QString m_newPassword;
    QString m_confirmPassword;
    QString m_lastGetTextPamMessage;
    QString m_lastLogin;
    QString m_lastPassword;

    void handleGetTextPamResponse(const char *response, bool echo, bool nonBlocking);
    void sendMessageToPam(const char *text, int tag);
    void sendDataToPam();
    void sendAuthDataToPam();
    void handleAuthError();
};

#endif // GREETERHANDLER_H
