#include "shutdownhandler.h"

#include "kdmshutdown.h"
#include "utils.h"
#include "greeterhandler.h"
#include <fly/dbus_iface.h>


ShutdownHandler::ShutdownHandler(GreeterHandler* greeterHandler, QObject* parent) :
    QObject(parent),
    m_greeterHandler(greeterHandler)
{
}

ShutdownHandler::~ShutdownHandler()
{
}

void ShutdownHandler::handleShutdown(const ShutdownAction action)
{
    switch (action) {
    case ShutdownAction::Reboot:
        if (checkShutdownPermission(SHUT_REBOOT)) {
            m_greeterHandler->doShutdown(SHUT_REBOOT);
        }
        break;
    case ShutdownAction::Shutdown:
        if (checkShutdownPermission(SHUT_HALT)) {
            m_greeterHandler->doShutdown(SHUT_HALT);
        }
        break;
    case ShutdownAction::ScheduleShutdown:
        KDMShutdown::scheduleShutdown();
        break;
    case ShutdownAction::Sleep:
    case ShutdownAction::SuspendToRam:
    case ShutdownAction::SuspendToDisk:
        m_greeterHandler->stop();
        const auto exitCode = KDMConfShutdown(-1, fetchSessions(lstRemote | lstTTY), SHUT_SUSPEND, QString()).exec();
        m_greeterHandler->start();
        if (exitCode) {
            char* error = nullptr;
            switch (action) {
            case ShutdownAction::Sleep:
                error = dbus_Logind_Suspend();
                break;
            case ShutdownAction::SuspendToRam:
                error = dbus_Logind_HybridSleep();
                break;
            case ShutdownAction::SuspendToDisk:
                error = dbus_Logind_Hibernate();
                break;
            default:
                break;
            }
            if (error) {
                free(error);
            }
        }
        break;
    }
}

bool ShutdownHandler::checkShutdownPermission(const int type, const QString& os)
{
    const auto sessions = fetchSessions(lstRemote | lstTTY);
    if (sessions.isEmpty() && _allowShutdown != SHUT_ROOT) {
        return true;
    }

    m_greeterHandler->stop();
    const auto exitCode = KDMConfShutdown(-1, sessions, type, os).exec();
    m_greeterHandler->start();
    if (exitCode == Schedule) {
        KDMShutdown::scheduleShutdown();
        return false;
    }
    return exitCode;
}
