#ifndef UIGREETER_H
#define UIGREETER_H

#include <QTimer>

#include "kfdialog.h"
#include "themeinfo.h"
#include "systeminfo.h"


class BaseThemeWidget;
class GreeterHandler;
class ShutdownHandler;
class GreeterManagers;
class StsSettings;


class UiGreeter : public FDialog
{
    Q_OBJECT

public:
    explicit UiGreeter(const ThemeInfo& themeInfo, const SystemInfo& systemInfo, QScreen* screen);
    ~UiGreeter() override;

public:
    void load();

public slots:
    int exec() override;

    void restartXServer();

protected:
    void paintEvent(QPaintEvent* event) override;
    void resizeEvent(QResizeEvent* event) override;
    bool eventFilter(QObject* watched, QEvent* event) override;
    void changeEvent(QEvent* event) override;

private slots:
    void performSuccessLogin();
    void exitConnection();
    void showConsoleDialog();
    void reloadToXDMCPDialog();
    void startGreeterFullTimeoutTimer();
    void onGreeterFullTimeoutTimerTimeout();
    void askPamForLastUserSession(const QString &userName);

private:
    void setupManagers();
    const QPixmap backgroundPixmap() const;

private:
    BaseThemeWidget* m_mainThemeWidget;
    SystemInfo m_systemInfo;
    ThemeInfo m_themeInfo;
    GreeterHandler* m_greeterHandler;
    ShutdownHandler* m_shutdownHandler;
    GreeterManagers* m_managers;
    StsSettings* m_stsSettings;
    bool m_wasPainted;
    QTimer m_greeterFullTimeoutTimer;

signals:
    void updateLastUserSession(const QString &sessionType);
};

#endif // UIGREETER_H
