/*

Copyright (C) 2005-2006 Oswald Buddenhagen <ossi@kde.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <QObject> //<klocale.h>
#include <QImage>
#include <QGuiApplication>
#include <QScreen>
#include <QProcess>
#include <QX11Info>

#include <QSvgRenderer>
#include <QPainter>
#include <QImage>

#include <fly/flymisc.h>
#include <fly/flyfileutils.h>
#include <fly/flystring.h>

#include "utils.h"
#include "kdm_greet.h"
#include "kdmconfig.h" //alex
#include "kdmpixmap.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/XKBlib.h>
#include <X11/extensions/XKBrules.h>

//alex
int xdpi = 0;
int faceSize = -1;
qint64 vkbdPid;
bool isTouch;

qint64 startVkbd()
{
    QStringList argList;
    char buf[100000];
    CmdLineInfo   clp;
    qint64 p=0;

    strncpy(buf,_vkbdCmd.toLocal8Bit().data(),sizeof(buf)-1); buf[sizeof(buf)-1]='\0';
    //fprintf(stderr,"vkbd call=%s\n",buf);
    if (MakeArgVector(buf, &clp)==0)
    {
      QString prog = QString::fromLocal8Bit(clp.argVector[0]);

      for (int i=1;i<clp.argCount;i++)
      {
        if (clp.argVector[i] && *(clp.argVector[i]))
        {
        //fprintf(stderr,"arg[%d]=%s\n",i,clp.argVector[i]);
          argList.append(QString::fromLocal8Bit(clp.argVector[i]));
        }
      }

      QProcess::startDetached(prog,argList,QString(),&p);
    }
    return p;
}

QString qString(char *str)
{
    if (!str)
        return QString();
    QString qs = QString::fromUtf8(str);
    free(str);
    return qs;
}

QStringList qStringList(char **strList)
{
    QStringList qsl;
    for (int i = 0; strList[i]; i++) {
        qsl.append(QString::fromUtf8(strList[i]));
        free(strList[i]);
    }
    free(strList);
    return qsl;
}

QList<DpySpec>
fetchSessions(int flags)
{
    QList<DpySpec> sessions;
    DpySpec tsess;

    gSet(1);
    gSendInt(G_List);
    gSendInt(flags);
  next:
    while (!(tsess.display = qString(gRecvStr())).isEmpty()) {
        tsess.from = qString(gRecvStr());
#ifdef HAVE_VTS
        tsess.vt = gRecvInt();
#endif
        tsess.user = qString(gRecvStr());
        tsess.session = qString(gRecvStr());
        tsess.flags = gRecvInt();
        if ((tsess.flags & isTTY) && !tsess.from.isEmpty())
            for (int i = 0; i < sessions.size(); i++)
                if (!sessions[i].user.isEmpty() &&
                    sessions[i].user == tsess.user &&
                    sessions[i].from == tsess.from)
                {
                    sessions[i].count++;
                    goto next;
                }
        tsess.count = 1;
        sessions.append(tsess);
    }
    gSet(0);
    return sessions;
}

void
decodeSession(const DpySpec &sess, QString &user, QString &loc)
{
    if (sess.flags & isTTY) {
        user =
            /*i18ncp( "user: ...", "%2: TTY login",*/ QObject::tr("%1: %2 TTY logins").arg(
                   _userList? sess.user : QObject::tr("Hidden username") ).arg(sess.count);
        loc =
#ifdef HAVE_VTS
            sess.vt ?
                QString("vt%1").arg(sess.vt) :
#endif
                !sess.from.isEmpty() ?
                    sess.from : sess.display;
    } else {
        user =
            sess.session.isEmpty() ?
                /*i18nc("... session",*/QObject::tr("Unused") :
                !sess.user.isEmpty() ?
                    /*i18nc("user: session type",*/QObject::tr("%1: %2").arg(
                          _userList? sess.user : QObject::tr("Hidden username"), sess.session) :
                    /*i18nc("... host",*/QObject::tr("X login on %1").arg( sess.session );
        loc =
#ifdef HAVE_VTS
            sess.vt ?
                QString("%1, vt%2").arg(sess.display).arg(sess.vt) :
#endif
                sess.display;
    }
}


std::string FlyDefaultLogoPath(std::string logoName, int desktopWidth, int desktopHeight, std::string logoColor)
{
    struct stat st;
    std::string logoPath;
    std::string logoSize; //logo standard sizes is 800, 1280, 1920, 2048 - proposed by designer Dm. Scherbakov
         if (desktopWidth<=800)  logoSize="800";
    else if (desktopWidth<=1280) logoSize="1280";
    else if (desktopWidth<=1920) logoSize="1920";
    else                         logoSize="2048";

    logoPath = logoName + "_"+logoSize + "_"+logoColor + ".png"; //1: full spec name_size_color.png
    if (stat(logoPath.c_str(),&st)==0) return logoPath;

    logoPath = logoName + "_"+logoSize + ".png"; //2: name_size.png
    if (stat(logoPath.c_str(),&st)==0) return logoPath;

    logoPath = logoName + "_"+logoColor + ".png"; //3: name_color.png
    if (stat(logoPath.c_str(),&st)==0) return logoPath;

    logoPath = logoName + ".png"; //4: name.png
    if (stat(logoPath.c_str(),&st)==0) return logoPath;

    return logoPath;
}

std::string FindFlyWallPaper(int screenWidth, int screenHeight, bool *exists)
{
    std::string name = "fly-default";
    std::string wpdir = Fly::concat(_dataDir.toStdString(),"/wallpapers");
    const char * pathToFind[2], *convertorFmt="fly-svg2png.sh %d %d %s %s";
    pathToFind[0]=wpdir.c_str();
    pathToFind[1]="/usr/share/wallpapers/fly-dm";

    if (exists) *exists=true;

    if (!fileExists(pathToFind[0],false))
    {
        if (mkdir(pathToFind[0],S_IRWXU) != 0) fprintf(stderr,"fly-qdm: failed to create dir '%s'\n", pathToFind[0]);
        sync();
    }

    int w, h;
    std::string ex=".png", path;
    int ratioWidth, ratioHeight;
    std::tie(ratioWidth, ratioHeight) = wallpaperSizeRatio(screenWidth, screenHeight);

    for (unsigned int i=0;i<sizeof(pathToFind)/sizeof(char *);i++)
    {
        w=screenWidth; h=screenHeight; //check for screen size WxH

        path = Fly::concat(pathToFind[i], '/', w, 'x', h, '/', name, ex);
        if (fileExists(path,true)) goto found;
        else fprintf(stderr, "fly-qdm: Can't find default wallpaper '%s' for screen %dx%d, attempt to look for other size\n", path.c_str(), w, h);

        w = ratioWidth; h = ratioHeight; //check for ratio W:H

        path = Fly::concat(pathToFind[i], '/', w, ':', h, '/', name, ex);
        if (fileExists(path,true)) goto found;
        else fprintf(stderr, "fly-qdm: Can't find default wallpaper '%s' for ratio %d:%d\n", path.c_str(), w, h);

        if (!ex.empty() && ex != ".svg")
        {
            path = Fly::concat(pathToFind[i], '/', w, ':', h, '/', name, ".svg");
            if (fileExists(path,true)) {
                std::string newPath = Fly::concat(_dataDir.toStdString(), "/wallpapers/", screenWidth, 'x', screenHeight);

                if (!fileExists(newPath,true)) {
                    if (mkdir(newPath.c_str(),S_IRWXU) != 0) fprintf(stderr,"fly-qdm: failed to create dir %s\n", newPath.c_str());
                    sync();
                }

                newPath += '/'; newPath += name; newPath += ".png";

                char cmd[PATH_MAX*2];
                int len = snprintf(cmd,sizeof(cmd)-1,convertorFmt,screenWidth,screenHeight,path.c_str(),newPath.c_str()); cmd[len]='\0';
                fprintf(stderr,"fly-qdm: SVG conversion by: %s... in progress\n",cmd);

                if (exists) *exists=false;
                system(cmd); //we need the result immediately

                if (fileExists(newPath,true)) {
                    path = std::move(newPath);
                    goto found;
                } else {
                    fprintf(stderr,"fly-qdm: Can't find converted wallpaper '%s', may be bash locked?\n", path.c_str());

                    fprintf(stderr,"fly-qdm: try convert wallpaper '%s' in fly-qdm\n", path.c_str());
                    QSvgRenderer renderer(QString::fromStdString(path));

                    QImage image(w, h, QImage::Format_ARGB32);
                    image.fill(0xaaA08080);

                    QPainter painter(&image);
                    renderer.render(&painter);

                    image.save(QString::fromStdString(newPath));

                    if (fileExists(newPath,true)) {
                        fprintf(stderr,"fly-qdm: Can't find converted wallpaper '%s' in fly-qdm\n", path.c_str());
                        path = std::move(newPath);
                        goto found;
                    }
                    return std::string();
                }
            } else {
                fprintf(stderr,"fly-qdm: Can't find default wallpaper '%s' for ratio %d:%d\n", path.c_str(), w, h);
            }
        }
    } //endfor
    return std::string();

found:
    return path;
}

QRect screenGeometry() 
{
    auto *screen = QGuiApplication::primaryScreen();
    const auto geometry = screen->geometry();
    return QRect(geometry.topLeft(), geometry.size() * screen->devicePixelRatio());
}

void GreyScaleImage(QImage *greyLogo)
{
      unsigned int *data = (unsigned int*)greyLogo->bits();
      int pixelCount = greyLogo->width() * greyLogo->height();
      // Convert each pixel to grayscale
      for(int i = 0; i < pixelCount; ++i) {
               int val = qGray(*data);
               *data = qRgba(val, val, val, qAlpha(*data));
               ++data;
      }
}

void setXWindowBackgroundPixmap(const QPixmap& pixmap)
{
    if (!QX11Info::isPlatformX11()) {
        return;
    }

    const auto display = QX11Info::display();
    const auto appScreen = QX11Info::appScreen();
    const auto depth = DefaultDepth(display, appScreen);
    const auto primariScreen = qApp->primaryScreen();
    const auto screenWidth = primariScreen->size().width() * primariScreen->devicePixelRatio();
    const auto screenHeight = primariScreen->size().height() * primariScreen->devicePixelRatio();

    const auto image = KdmPixmap::ximageFromQImage(display,
                                        appScreen,
                                        depth,
                                        pixmap.toImage().scaled(screenWidth,
                                                                screenHeight,
                                                                Qt::KeepAspectRatio,
                                                                Qt::SmoothTransformation));

    if (!image) {
        return;
    }

    const auto rootWindow = QX11Info::appRootWindow();
    const auto rootPixmap = XCreatePixmap(display, rootWindow, screenWidth, screenHeight, depth);
    const auto gc = DefaultGC(display, appScreen);
    XPutImage(display, rootPixmap, gc, image, 0, 0, 0, 0, screenWidth, screenHeight);

    XSetWindowBackgroundPixmap(display, rootWindow, rootPixmap);
    XClearWindow(display, rootWindow);

    XDestroyImage(image);
    XFreePixmap(display, rootPixmap);
}

QString getXKBGroupName(int gr, bool cached)
{
  static Atom _XA_XKB_RF_NAMES_PROP_ATOM=None;
  static QString  xkbGroupName[4];
  static int      xkbGroupNum=-1;

  Atom real_prop_type;
  int fmt;
  unsigned long nitems;
  unsigned long extra_bytes;
  unsigned char *prop_data = nullptr;

  /* dont use cache due to it may be changed by setxkbmap */
  if (cached && xkbGroupNum!=-1) {
      if (gr>=0 && gr<xkbGroupNum) return xkbGroupName[gr];
      else return QString();
  }
  for (int i=0;i<xkbGroupNum;i++) xkbGroupName[i].clear();
  xkbGroupNum=0;

  const auto display = QX11Info::display();
  const auto rootWindow = QX11Info::appRootWindow();

  if (_XA_XKB_RF_NAMES_PROP_ATOM==None) _XA_XKB_RF_NAMES_PROP_ATOM= XInternAtom(display, _XKB_RF_NAMES_PROP_ATOM, False);

  if (_XA_XKB_RF_NAMES_PROP_ATOM!=None &&
       XGetWindowProperty(display, rootWindow, _XA_XKB_RF_NAMES_PROP_ATOM, 0L, _XKB_RF_NAMES_PROP_MAXLEN,
		False, XA_STRING, &real_prop_type, &fmt, &nitems, &extra_bytes,
		( unsigned char ** ) ( void * ) & prop_data)==Success &&
        extra_bytes==0 && real_prop_type==XA_STRING && fmt==8)
  {
      char *names[5] = {nullptr,nullptr,nullptr,nullptr,nullptr}; //all fields
      char *p = reinterpret_cast<char *>(prop_data);

      char *end = p+nitems;
      for (int i=0;p<end;i++) { names[i]=p; p += strlen(p)+1; }

      if (names[2]) {
          QStringList layouts = QString::fromLocal8Bit(names[2]).split(QString(","));
          for (auto l : layouts) xkbGroupName[xkbGroupNum++] = l;
      }
  }
  if (prop_data) XFree(prop_data);

  if (gr>=0 && gr<xkbGroupNum) return xkbGroupName[gr];
  return QString();
}

