#ifndef THEMEWIDGET_H
#define THEMEWIDGET_H

#include "widgets/basethemewidget.h"
#include "widgets/datetimewidgethandler.h"
#include "managers/usersmanager.h"
#include "managers/localauthmanager.h"
#include "themesettings.h"

#include <QTimer>
#include <QMenu>


class CommonWidget;
class LoginWidget;
class UserCellWidget;
class HorizontalUserWidget;
class VirtualKeyboardManager;
class KeyboardManager;
class ShutdownManager;
class SessionsManager;
class ConnectionManager;
class ConsoleAuthManager;
class DomainManager;
class BackgroundWidget;


class ThemeWidget : public BaseThemeWidget
{
    Q_OBJECT
//    Q_PLUGIN_METADATA(IID BaseThemeWidget_iid)
//    Q_INTERFACES(BaseThemeWidget)

public:
    explicit ThemeWidget(QWidget* parent = nullptr);
    ~ThemeWidget() override;

public:
    void load(const ThemeDescription& themeDescription, GreeterManagers* managers, const EnvironmentData& environment) override;
    void exec() override;
    const QPixmap backgroundPixmap() const override;

protected slots:
    void performLoginRequest(const AuthData& data);
    void performLoginFailed();
    void performSuccessLogin();
    void loginFailedTimerTimeout();
    void virtualKeyboardErrorHandle(const QString& error);
    void capsLockStateChangedHandle(const bool lock);
    void rebootHandle();
    void shutdownHandle();
    void sleepHandle();
    void suspendToRamHandle();
    void consoleLoginhandle();
    void exitActionTriggered();
    void changeAuthPageActionTriggered();
    void authPageChanged();
    void openNewPasswordDialog(const bool withAuth);
    void showErrorMessage(const QString &message);
    void showMessage(const QString &message);

protected:
    template<typename T>
    T findWidget(const QString& name) const;
    template<typename T>
    T findManager(const QString& id) const;

    void loadTranslations();

    void resizeEvent(QResizeEvent* event) override;

    void checkNoPassSettings();

protected:
    BackgroundWidget* m_backgroundWidget;

    ThemeSettings m_settings;
    EnvironmentData m_environment;
    ThemeDescription m_themeDescription;
    DateTimeWidgetHandler m_dateTimeWidget;
    UsersManager* m_usersManager;
    LocalAuthManager* m_localAuthManager;
    VirtualKeyboardManager* m_virtualKeyboardManager;
    KeyboardManager* m_keyboardManager;
    ShutdownManager* m_shutdownManager;
    SessionsManager* m_sessionsManager;
    ConnectionManager* m_connectionManager;
    ConsoleAuthManager* m_consoleAuthManager;
    DomainManager *m_domainManager;
    CommonWidget* m_commonWidget;
    LoginWidget* m_loginWidget;
    QTimer m_loginFailedTimer;
    QAction* m_changeAuthPageAction;

private:
    QMenu *createSelectSessionMenu();

    bool m_isNewPasswordDialogOpened = false;

signals:
    void newPasswordInsertionCompleted(const QString &login,
                                       const QString &oldPassword,
                                       const QString &newPassword,
                                       const QString &confirmPassword);
    void dpyMenuUpdatingCompleted();

private slots:
    void onNewPasswordInsertionCompleted(const QString &oldPassword,
                                         const QString &newPassword,
                                         const QString &confirmPassword);
};

class PluginThemeLoader: public AbstractPluginThemeLoader
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID AbstractPluginThemeLoader_iid)
    Q_INTERFACES(AbstractPluginThemeLoader)

public:
    explicit PluginThemeLoader(QObject* parent = nullptr);
    virtual ~PluginThemeLoader();

    BaseThemeWidget* createTheme() const override;
};

#endif // THEMEWIDGET_H
