#!/bin/bash

at_exit() {
    [ -n "${TMPFILE}" ] && rm "${TMPFILE}"
}

max() {
    echo $(($1>$2?$1:$2))
}

calculate_dpi() {
    local screen_size
    local result
    local default_dpi=96
    local calculated_dpi
    local current_resolution
    local rotated
    local x_resolution
    local y_resolution

    #try to get size of primary monitor
    current_resolution=$(xrandr 2>/dev/null | grep -Eo 'primary [1-9][0-9]{1,3}x[1-9][0-9]{1,3}' | grep -Eo '[1-9][0-9]{1,3}x[1-9][0-9]{1,3}')

    if [ -n "$current_resolution" ]; then
      screen_size=$(xrandr --current 2>/dev/null | grep -A1 primary | grep -Eo '[1-9][0-9]{1,3}mm x [1-9][0-9]{1,3}mm')

      rotated=$(xrandr 2>/dev/null | grep 'primary' | grep 'left (')
      if [ ! -n "$rotated" ]; then
        rotated=$(xrandr 2>/dev/null | grep 'primary' | grep 'right (')
      fi
    else
      #if no primary then get size of the first monitor
      current_resolution=$(xrandr 2>/dev/null | grep -Eo 'current [1-9][0-9]{1,3} x [1-9][0-9]{1,3}' | grep -Eo '[1-9][0-9]{1,3} x [1-9][0-9]{1,3}')
      screen_size=$(xrandr --current 2>/dev/null | grep -A1 current | grep -Eo '[1-9][0-9]{1,3}mm x [1-9][0-9]{1,3}mm')

      rotated=$(xrandr 2>/dev/null | grep 'current' | grep 'left (')
      if [ ! -n "$rotated" ]; then
        rotated=$(xrandr 2>/dev/null | grep 'current' | grep 'right (')
      fi
    fi

    x_resolution=$(echo "$current_resolution" | grep -Eo '^[1-9][0-9]{1,3}')
    y_resolution=$(echo "$current_resolution" | grep -Eo  '[1-9][0-9]{1,3}$')

    #Check if we have screen_size from xrandr output
    if [ -n "$screen_size" ]; then
        local x_size=$(echo "$screen_size" | grep -Eo '^[1-9][0-9]{1,3}')
        local y_size=$(echo "$screen_size" | grep -Eo  '[1-9][0-9]{1,3}mm$' | grep -Eo '^[1-9][0-9]{1,3}')

        if [ -n "$rotated" ]; then
          local tmp_size="$x_size"
          x_size="$y_size"
          y_size="$tmp_size"
        fi

        local x_dpi=$((($x_resolution * 254) / ($x_size * 10)))
        local y_dpi=$((($y_resolution * 254) / ($y_size * 10)))
        calculated_dpi=$(max $x_dpi $y_dpi)
        if [[ "$x_size" -gt "220" && "$x_size" -le "320" && "$x_dpi" -le "200" ]]; then
          calculated_dpi="$default_dpi"
        fi
    else
        calculated_dpi=$default_dpi
        if [ x"$1" == x"user" ]; then
          #dont ask user about monitor size, inherit from fly-dm
          if [ -e /etc/X11/fly-dm/dpi ]; then
            calculated_dpi=`cat /etc/X11/fly-dm/dpi`
          fi
        else
          #screen_size is empty. Let's assume DPI is 96
          calculated_dpi=$default_dpi
          #echo "DPI calculation failed. Using default value $default_dpi." 1>&2
          #fly-qmsg -s -l 3000 -i warning "DPI calculation failed. Using default value $default_dpi." &
          export LANG=ru_RU.UTF-8
          diag=$(QT_STYLE_OVERRIDE=FlyPlastiqueTablet fly-dialog --title "Диагональ экрана" --spinbox "Диагональ экрана в дюймах" 10 4 100 1)
          calculated_dpi=$(echo "sqrt($x_resolution^2+$y_resolution^2)/$diag" | bc)
          if [[ "$diag" -gt "10" && "$diag" -le "14" && "$calculated_dpi" -le "200" ]]; then
            calculated_dpi="$default_dpi"
          fi
        fi
    fi

    if [ "$calculated_dpi" -lt "120" ]; then
      #echo "Calculated DPI value is less than 120. Using default value $default_dpi." 1>&2
      result=$default_dpi
    else
      result=$calculated_dpi
    fi

    FMS_DPI=$result
    export FMS_DPI
}

get_dpi() {
    local dpi_file
    local dpi_file_dir

    dpi_file_dir=/etc/X11/fly-dm
    if [ ! -d $dpi_file_dir ]; then mkdir -p $dpi_file_dir; fi

    dpi_file="${dpi_file_dir}/dpi"

    if [ -e $dpi_file ]; then
        cat $dpi_file
    else
        calculate_dpi
        echo $FMS_DPI > $dpi_file
        echo $FMS_DPI
    fi
}

#check if display is local non-nested, i.e. display name starts with ':'
if [ "${DISPLAY:0:1}" != ":" ]; then exit; fi
#check if nested
fly-wmfunc FLYWM_IS_NESTED_SESSION
if [ "$?" = 0 ]; then exit; fi

#ask dpi if touchscreen only?
#fly-wmfunc FLYWM_GET_TOUCHSCREEN > /dev/null
#if [ $? -ne "0" ]; then exit; fi

#ask dpi if no VM?
systemd-detect-virt > /dev/null
if [ $? -eq "0" ]; then exit; fi


#get dpi
if [ x"$1" == x"user" ]; then
  #get dpi for current users's primary monitor
  calculate_dpi user
  dpi=$FMS_DPI
else
  #get dpi for current primary monitor and store it in /etc/X11/fly-dm/dpi
  dpi=$(get_dpi)
fi
#echo primary monitor dpi = $dpi

#apply font dpi if required
if [ "$dpi" -gt "120" ]; then
  xft_dpi="Xft.dpi: $dpi"
  echo "${xft_dpi}" | /usr/bin/xrdb -merge -quiet
fi

#apply cursor size always to get right size for primary screen
if [ "$dpi" -le "120" ]; then
    cursor_size=16
elif [ "$dpi" -le "140" ]; then
    cursor_size=24
elif [ "$dpi" -le "200" ]; then
    cursor_size=32
elif [ "$dpi" -le "300" ]; then
    cursor_size=48
else
    cursor_size=64
fi
echo "Xcursor.size: $cursor_size" | xrdb -merge -quiet
